#ifndef __SHA256_H
#define __SHA256_H

#define DoMemZero(dest,len) (memset((dest), 0, (len)))

#if SIZEOF_LONG > 4
	#define TRUNCLONG(x) ((x) & IL_MAX_UINT32)
	#define ROTATE(x,n) (TRUNCLONG(((x) >> (n))) | ((x) << (32 - (n))))
	#define SHIFT(x,n) (TRUNCLONG(((x) >> (n))))
#else
	#define TRUNCLONG(x) (x)
	#define ROTATE(x,n) (((x) >> (n)) | ((x) << (32 - (n))))
	#define SHIFT(x,n) ((x) >> (n))
#endif

#define CH(x,y,z) (((x) & (y)) ^ (TRUNCLONG(~(x)) & (z)))
#define MAJ(x,y,z) (((x) & (y)) ^ ((x) & (z)) ^ ((y) & (z)))
#define SUM0(x) (ROTATE((x), 2) ^ ROTATE((x), 13) ^ ROTATE((x), 22))
#define SUM1(x) (ROTATE((x), 6) ^ ROTATE((x), 11) ^ ROTATE((x), 25))
#define RHO0(x) (ROTATE((x), 7) ^ ROTATE((x), 18) ^ SHIFT((x), 3))
#define RHO1(x) (ROTATE((x), 17) ^ ROTATE((x), 19) ^ SHIFT((x), 10))

#define SHA256_HASH_SIZE 32

typedef struct _tagSHA256Context {
	unsigned char input[64];
	u_int32_t inputLen;
	u_int32_t A, B, C, D, E, F, G, H;
	u_int64_t totalLen;
} SHA256Context;

static u_int32_t const K[64] = {
	0x428a2f98, 0x71374491, 0xb5c0fbcf, 0xe9b5dba5,
	0x3956c25b, 0x59f111f1, 0x923f82a4, 0xab1c5ed5,
	0xd807aa98, 0x12835b01, 0x243185be, 0x550c7dc3,
	0x72be5d74, 0x80deb1fe, 0x9bdc06a7, 0xc19bf174,
	0xe49b69c1, 0xefbe4786, 0x0fc19dc6, 0x240ca1cc,
	0x2de92c6f, 0x4a7484aa, 0x5cb0a9dc, 0x76f988da,
	0x983e5152, 0xa831c66d, 0xb00327c8, 0xbf597fc7,
	0xc6e00bf3, 0xd5a79147, 0x06ca6351, 0x14292967,
	0x27b70a85, 0x2e1b2138, 0x4d2c6dfc, 0x53380d13,
	0x650a7354, 0x766a0abb, 0x81c2c92e, 0x92722c85,
	0xa2bfe8a1, 0xa81a664b, 0xc24b8b70, 0xc76c51a3,
	0xd192e819, 0xd6990624, 0xf40e3585, 0x106aa070,
	0x19a4c116, 0x1e376c08, 0x2748774c, 0x34b0bcb5,
	0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f, 0x682e6ff3,
	0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208,
	0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2
};

static void ProcessBlock(SHA256Context *sha, const unsigned char *block);
void SHA256Data(SHA256Context *sha, const void *buffer, unsigned long len);
void SHA256Finalize(SHA256Context *sha, unsigned char hash[SHA256_HASH_SIZE]);
void SHA256Init(SHA256Context *sha);
static void WriteLong(unsigned char *buf, u_int32_t value);
extern unsigned char* get_sha256_hash(const char* unhashed);
extern char* get_sha256_hash_hex_str(const char* unhashed);

#endif
